<?php

use ConnectpayPaymentGateway\Includes\CpConstants;

class WC_CP_Payment_Gateway extends WC_Payment_Gateway {

	const PAYMENT_COUNTRY_CODE_LITHUANIA = 'LT';
	const PAYMENT_COUNTRY_CODE_GERMANY = 'DE';
	const PAYMENT_COUNTRY_CODE_FINLAND = 'FI';
	const PAYMENT_COUNTRY_CODE_NETHERLANDS = 'NL';
	const PAYMENT_COUNTRY_CODE_ALL = 'all';
	const CHECKBOX_FALSE = 'no';
	const CHECKBOX_TRUE = 'yes';
	const PAYMENT_METHOD_PIS = 'PIS';
	const PAYMENT_METHOD_CARD = 'CARD';
	const API_UPDATE_INTERVAL = 10;
	const PAYMENT_COUNTRY_TITLE_FOR_ALL = 'All available countries';

	public function __construct() {
		$this->id                 = 'cp_payment_gateway';
		$this->icon               = apply_filters( 'woocommerce_cp_icon',
			plugins_url() . '/connectpay-payment-gateway/assets/img/logo.png' );
		$this->has_fields         = true;
		$this->method_title       = _x( 'ConnectPay', 'ConnectPay Payment Method', CpConstants::CP_TRANSLATIONS );
		$this->method_description = __( 'ConnectPay payment gateway.', CpConstants::CP_TRANSLATIONS );

		$this->payment_methods = get_option(
			'cp_payment_methods',
			[
				'woocommerce_cp_payment_gateway_method_pis'  => 'yes',
				'woocommerce_cp_payment_gateway_method_card' => 'yes',
			]
		);

		$this->payment_providers_db = get_option( 'cp_payment_providers', [] );

		$this->payment_country = get_option(
			'cp_payment_country',
			self::PAYMENT_COUNTRY_CODE_LITHUANIA
		);

		$this->is_all_countries = ! empty( get_option(
			'cp_is_all_countries',
			''
		) );

		$this->last_update = get_option( 'cp_last_update', '0000-00-00 00:00:00' );

		$this->init_form_fields();
		$this->init_settings();

		$this->title       = $this->get_option( 'title' );
		$this->description = $this->get_option( 'description' );
		$this->enabled     = $this->get_option( 'enabled' );

		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, [ $this, 'process_admin_options' ] );
		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, [ $this, 'save_payment_methods' ] );
		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, [ $this, 'save_payment_providers' ] );
		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, [ $this, 'save_payment_country' ] );
		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, [ $this, 'handle_file_upload' ] );

		add_action( 'wp_enqueue_scripts', [ $this, 'payment_style_scripts' ] );
		add_action( 'admin_enqueue_scripts', [ $this, 'load_admin_scripts' ] );
	}

	public function init_form_fields() {
		$this->form_fields = [
			'enabled'                  => [
				'title'   => __( 'Enable/Disable', CpConstants::CP_TRANSLATIONS ),
				'type'    => 'checkbox',
				'label'   => __( 'Enable ConnectPay Payment', CpConstants::CP_TRANSLATIONS ),
				'default' => 'yes',
			],
			'title'                    => [
				'title'       => __( 'Title', 'woocommerce' ),
				'type'        => 'text',
				'description' => __( 'This controls the title which the user sees during checkout.', CpConstants::CP_TRANSLATIONS ),
				'default'     => _x( 'ConnectPay', 'ConnectPay Payment Method', CpConstants::CP_TRANSLATIONS ),
				'desc_tip'    => true,
			],
			'cp_client_id'             => [
				'title'       => __( 'Client ID:', CpConstants::CP_TRANSLATIONS ),
				'type'        => 'password',
				'description' => __( 'Client ID retrieved from ConnectPay', CpConstants::CP_TRANSLATIONS ),
				'default'     => '',
				'desc_tip'    => true,
			],
			'cp_client_secret'         => [
				'title'       => __( 'Client Secret:', CpConstants::CP_TRANSLATIONS ),
				'type'        => 'password',
				'description' => __( 'Client Secret retrieved from ConnectPay', CpConstants::CP_TRANSLATIONS ),
				'default'     => '',
				'desc_tip'    => true,
			],
			'cp_brand_id'              => [
				'title'       => __( 'Brand ID:', CpConstants::CP_TRANSLATIONS ),
				'type'        => 'password',
				'description' => __( 'Brand ID retrieved from ConnectPay', CpConstants::CP_TRANSLATIONS ),
				'default'     => '',
				'desc_tip'    => true,
			],
			'private_key'              => [
				'type' => 'private_key',
			],
			'payment_methods'          => [
				'type' => 'payment_methods',
			],
			'update_payment_providers' => [
				'type' => 'update_payment_providers',
			],
			'last_update'              => [
				'type' => 'last_update',
			],
			'payment_providers'        => [
				'type' => 'payment_providers',
			],
			'payment_country'          => [
				'type' => 'payment_country',
			],
		];

		if ( 'DE' === $this->payment_country || $this->is_all_countries ) {
			unset( $this->form_fields['payment_providers'] );
			unset( $this->form_fields['update_payment_providers'] );
		}
	}

	public function load_admin_scripts() {
		$this->check_update_providers_error();
	}

	public function payment_fields() {
		wp_nonce_field( 'checkout_nonce', 'checkout_wpnonce' );
		$translations                  = $this->get_cp_checkout_description_translations();
		$payment_methods_options       = get_payment_methods_options_list( $this->payment_methods );
		$payment_provider_options      = get_payment_providers_options_list(
			$this->payment_providers_db, $payment_methods_options, $translations
		);
		$payment_provider_options_html = get_payment_providers_options_list_html( $payment_provider_options );

		echo wp_kses( $payment_provider_options_html, array_merge(
			[
				'input' => [
					'type'  => true,
					'value' => true,
					'id'    => true,
					'class' => true,
					'name'  => true,
				],
			],
			wp_kses_allowed_html( 'post' )
		) );

		echo wp_kses_post( wpautop( '<div id="cp_description">' . $translations['description'] . '</div>' ) );
	}

	public function payment_style_scripts() {
		if ( ! is_cart() && ! is_checkout() && ! isset( $_GET['pay_for_order'] ) ) {
			return;
		}

		if ( 'no' === $this->enabled ) {
			return;
		}

		wp_register_style( 'cp_checkout_css', plugins_url( '../assets/css/cp-styles.css', __FILE__ ), [], rand( 111, 9999 ), 'all' );
		wp_enqueue_style( 'cp_checkout_css' );
	}

	// Admin scripts
	private function check_update_providers_error() {
		if ( isset( $_GET['providersupdate'], $_COOKIE['request_error'] ) && ! empty( $_GET['providersupdate'] ) && 'failed' === $_GET['providersupdate'] ) {
			add_action( 'admin_notices', 'get_providers_error' );

			function get_providers_error() {
				$error = sanitize_text_field( $_COOKIE['request_error'] );
				?>
                <div class="error notice">
                    <p><?php
						esc_attr_e( 'Error updating payment providers: ' . $error, CpConstants::CP_TRANSLATIONS ); ?></p>
                </div>
				<?php
			}
		}
	}

	public function get_cp_checkout_description_translations() {
		$locale                        = get_locale();
		$translations                  = get_option( 'cp_checkout_translations' );
		$descriptions                  = $translations['description'];
		$by_card_translations          = $translations['by_card'];
		$by_bank_transfer_translations = $translations['by_bank_transfer'];

		$description = array_key_exists( $locale, $descriptions ) ? $descriptions[ $locale ] : $descriptions['en_US'];

		$by_card_translation = array_key_exists( $locale, $by_card_translations )
			? $by_card_translations[ $locale ]
			: $by_card_translations['en_US'];

		$by_bank_transfer_translation = array_key_exists( $locale, $by_bank_transfer_translations )
			? $by_bank_transfer_translations[ $locale ]
			: $by_bank_transfer_translations['en_US'];

		$description_html = $description['main_part'] . ' <a target="_blank" href="' . $description['terms_of_use_url'] . '" class="cp-hyperlink">' . $description['terms_of_use']
		                    . '</a> ' . $description['and'] . ' <a target="_blank" href="' . $description['privacy_policy_url'] . '" class="cp-hyperlink">'
		                    . $description['privacy_policy'] . '</a>';

		$translations_in_html = [
			'description'      => $description_html,
			'by_card'          => $by_card_translation['main_part'],
			'by_bank_transfer' => $by_bank_transfer_translation['main_part'],
		];

		return $translations_in_html;
	}

	//<editor-fold desc="Payment gateway needed code fragments">

	/**
	 * Check If The Gateway Is Available For Use.
	 *
	 * @return bool
	 */
	public function is_available() {
		$order          = null;
		$needs_shipping = false;

		// Test if shipping is needed first.
		if ( WC()->cart && WC()->cart->needs_shipping() ) {
			$needs_shipping = true;
		} elseif ( is_page( wc_get_page_id( 'checkout' ) ) && 0 < get_query_var( 'order-pay' ) ) {
			$order_id = absint( get_query_var( 'order-pay' ) );
			$order    = wc_get_order( $order_id );

			// Test if order needs shipping.
			if ( $order && 0 < count( $order->get_items() ) ) {
				foreach ( $order->get_items() as $item ) {
					$_product = $item->get_product();
					if ( $_product && $_product->needs_shipping() ) {
						$needs_shipping = true;
						break;
					}
				}
			}
		}

		$needs_shipping = apply_filters( 'woocommerce_cart_needs_shipping', $needs_shipping );

		// Only apply if all packages are being shipped via chosen method, or order is virtual.
		if ( ! empty( $this->enable_for_methods ) && $needs_shipping ) {
			$order_shipping_items            = is_object( $order ) ? $order->get_shipping_methods() : false;
			$chosen_shipping_methods_session = WC()->session->get( 'chosen_shipping_methods' );

			if ( $order_shipping_items ) {
				$canonical_rate_ids = $this->get_canonical_order_shipping_item_rate_ids( $order_shipping_items );
			} else {
				$canonical_rate_ids = $this->get_canonical_package_rate_ids( $chosen_shipping_methods_session );
			}

			if ( ! count( $this->get_matching_rates( $canonical_rate_ids ) ) ) {
				return false;
			}
		}

		return parent::is_available();
	}

	/**
	 * Checks to see whether or not the admin settings are being accessed by the current request.
	 *
	 * @return bool
	 */
	private function is_accessing_settings() {
		if ( is_admin() ) {
			// phpcs:disable WordPress.Security.NonceVerification
			if ( ! isset( $_REQUEST['page'] ) || 'wc-settings' !== $_REQUEST['page'] ) {
				return false;
			}
			if ( ! isset( $_REQUEST['tab'] ) || 'checkout' !== $_REQUEST['tab'] ) {
				return false;
			}
			if ( ! isset( $_REQUEST['section'] ) || 'cp_payment_gateway' !== $_REQUEST['section'] ) {
				return false;
			}

			// phpcs:enable WordPress.Security.NonceVerification

			return true;
		}

		return false;
	}

	/**
	 * Loads all of the shipping method options for the enable_for_methods field.
	 *
	 * @return array
	 */
	private function load_shipping_method_options() {
		// Since this is expensive, we only want to do it if we're actually on the settings page.
		if ( ! $this->is_accessing_settings() ) {
			return [];
		}

		$data_store = WC_Data_Store::load( 'shipping-zone' );
		$raw_zones  = $data_store->get_zones();

		foreach ( $raw_zones as $raw_zone ) {
			$zones[] = new WC_Shipping_Zone( $raw_zone );
		}

		$zones[] = new WC_Shipping_Zone( 0 );

		$options = [];
		foreach ( WC()->shipping()->load_shipping_methods() as $method ) {
			$options[ $method->get_method_title() ] = [];

			// Translators: %1$s shipping method name.
			$options[ $method->get_method_title() ][ $method->id ] = sprintf( __( 'Any &quot;%1$s&quot; method', 'woocommerce' ),
				$method->get_method_title() );

			foreach ( $zones as $zone ) {
				$shipping_method_instances = $zone->get_shipping_methods();

				foreach ( $shipping_method_instances as $shipping_method_instance_id => $shipping_method_instance ) {
					if ( $shipping_method_instance->id !== $method->id ) {
						continue;
					}

					$option_id = $shipping_method_instance->get_rate_id();

					// Translators: %1$s shipping method title, %2$s shipping method id.
					$option_instance_title = sprintf( __( '%1$s (#%2$s)', 'woocommerce' ), $shipping_method_instance->get_title(),
						$shipping_method_instance_id );

					// Translators: %1$s zone name, %2$s shipping method instance name.
					$option_title = sprintf( __( '%1$s &ndash; %2$s', 'woocommerce' ),
						$zone->get_id() ? $zone->get_zone_name() : __( 'Other locations', 'woocommerce' ), $option_instance_title );

					$options[ $method->get_method_title() ][ $option_id ] = $option_title;
				}
			}
		}

		return $options;
	}

	/**
	 * Converts the chosen rate IDs generated by Shipping Methods to a canonical 'method_id:instance_id' format.
	 *
	 * @param array $order_shipping_items Array of WC_Order_Item_Shipping objects.
	 *
	 * @return array $canonical_rate_ids    Rate IDs in a canonical format.
	 * @since  3.4.0
	 *
	 */
	private function get_canonical_order_shipping_item_rate_ids( $order_shipping_items ) {
		$canonical_rate_ids = [];

		foreach ( $order_shipping_items as $order_shipping_item ) {
			$canonical_rate_ids[] = $order_shipping_item->get_method_id() . ':' . $order_shipping_item->get_instance_id();
		}

		return $canonical_rate_ids;
	}

	/**
	 * Converts the chosen rate IDs generated by Shipping Methods to a canonical 'method_id:instance_id' format.
	 *
	 * @param array $chosen_package_rate_ids Rate IDs as generated by shipping methods. Can be anything if a shipping method doesn't honor
	 *     WC conventions.
	 *
	 * @return array $canonical_rate_ids  Rate IDs in a canonical format.
	 * @since  3.4.0
	 *
	 */
	private function get_canonical_package_rate_ids( $chosen_package_rate_ids ) {
		$shipping_packages  = WC()->shipping()->get_packages();
		$canonical_rate_ids = [];

		if ( ! empty( $chosen_package_rate_ids ) && is_array( $chosen_package_rate_ids ) ) {
			foreach ( $chosen_package_rate_ids as $package_key => $chosen_package_rate_id ) {
				if ( ! empty( $shipping_packages[ $package_key ]['rates'][ $chosen_package_rate_id ] ) ) {
					$chosen_rate          = $shipping_packages[ $package_key ]['rates'][ $chosen_package_rate_id ];
					$canonical_rate_ids[] = $chosen_rate->get_method_id() . ':' . $chosen_rate->get_instance_id();
				}
			}
		}

		return $canonical_rate_ids;
	}

	/**
	 * Indicates whether a rate exists in an array of canonically-formatted rate IDs that activates this gateway.
	 *
	 * @param array $rate_ids Rate ids to check.
	 *
	 * @return array
	 * @since  3.4.0
	 *
	 */
	private function get_matching_rates( $rate_ids ) {
		// First, match entries in 'method_id:instance_id' format. Then, match entries in 'method_id' format by stripping off the instance ID from the candidates.
		return array_unique( array_merge( array_intersect( $this->enable_for_methods, $rate_ids ),
			array_intersect( $this->enable_for_methods, array_unique( array_map( 'wc_get_string_before_colon', $rate_ids ) ) ) ) );
	}
	//</editor-fold>

	/**
	 * Process the payment and return the result.
	 *
	 * @param int $order_id Order ID.
	 *
	 * @return array
	 */
	public function process_payment( $order_id ) {
		$order                 = wc_get_order( $order_id );
		$method                = null;
		$payment_provider      = self::PAYMENT_METHOD_PIS;
		$callback              = get_home_url() . '/?rest_route=/cp/callback&order=' . $order_id;
		$filtered_phone_number = preg_replace( '/[^0-9]/', '', $order->get_billing_phone() );

		/*
		 * Examples of cp_payment_provider values:
		 * - woocommerce_cp_payment_gateway_method_pis (for DE users and PIS method)
		 * - woocommerce_cp_payment_gateway_method_card (for CARD method)
		 * - woocommerce_cp_payment_gateway_provider_6f1ef91e-4d86-4e90-a1e6-fdea39cd04fd (for PIS method)
		 */
		if ( isset( $_POST['checkout_wpnonce'] ) && wp_verify_nonce( sanitize_key( $_POST['checkout_wpnonce'] ), 'checkout_nonce' ) ) {
			if ( isset( $_POST['cp_payment_provider'] ) && ! empty( sanitize_text_field( $_POST['cp_payment_provider'] ) ) ) {
				$provider_value   = sanitize_text_field( $_POST['cp_payment_provider'] );
				$payment_provider = explode( '_', $provider_value )[5];
				$method           = 'card' === $payment_provider ? self::PAYMENT_METHOD_CARD : self::PAYMENT_METHOD_PIS;
			}
		}

		$body = [
			'callback'  => $callback,
			'firstName' => $order->get_billing_first_name(),
			'lastName'  => $order->get_billing_last_name(),
			'email'     => $order->get_billing_email(),
			'phone'     => $filtered_phone_number,
			'total'     => $order->get_total(),
			'currency'  => $order->get_currency(),
			'method'    => $method,
			'orderId'   => strval( $order_id ),
			'ip'        => $order->get_customer_ip_address(),
		];

		if ( 'card' !== $payment_provider && 'pis' !== $payment_provider ) {
			$body['providerId'] = $payment_provider;
		}

		/*
		* API-function of ConnectPay
		* It sends the details of the order and initiates the payment
		* If the data is processed, return URL of Payment page at bank's website and payment ID.
		*/
		$payment = $this->initiate_cp_payment( $body );

		$logger = wc_get_logger();
		$logger->info(
			print_r( [
				'type'         => 'Place Order',
				'payment_data' => $payment,
				'body'         => $body,
			], true )
		);

		if ( isset( $payment['error'] ) && ! empty( $payment['error'] ) ) {
			$currency_error_code = '20537';
			$response_text       = trim( substr( $payment['error'], strpos( $payment['error'], '{' ) ), '()' );
			$is_currency_error   = $this->cp_get_payment_response_error_parameter(
				$response_text,
				'code'
			);

			if ( $is_currency_error === $currency_error_code ) {
				wc_add_notice( sprintf(__( '%s is not available for selected payment method', CpConstants::CP_TRANSLATIONS ), $order->get_currency() ), 'error' );
			} else {
				wc_add_notice( __( 'Payment error', CpConstants::CP_TRANSLATIONS ), 'error' );
			}
		} else {
			$paymentId = $payment['payment']['paymentId'];
			$redirect  = $payment['payment']['redirectUrl'];

			// Update order status
			//	        $order->update_status('wc-pending', __('Processing payment via ConnectPay', CpConstants::CP_TRANSLATIONS));

			// Set META parameters
			update_post_meta( $order_id, 'cp_payment_id', $paymentId );
			update_post_meta( $order_id, 'cp_is_terminal', 'false' );
			update_post_meta( $order_id, 'cp_order_success_url', $this->get_return_url( $order ) );
		}

		return [
			'result'   => 'success',
			'redirect' => $redirect,
		];
	}

	private function cp_get_payment_response_error_parameter( $body, $param ) {
		$decoded_body = json_decode( substr( $body, strpos( $body, '{' ) ), true );

		return $decoded_body['messages'][0][ $param ];
	}

	private function initiate_cp_payment( array $body ) {
		$payment_details = [];
		$payment_details = apply_filters( 'cp_initiate_payment', $payment_details, $body );

		return $payment_details;
	}

	/**
	 * Output for the order received page.
	 */
	public function thankyou_page() {
		if ( $this->instructions ) {
			echo wp_kses_post( wpautop( wptexturize( $this->instructions ) ) );
		}
	}

	private function generate_cp_admin_nonce() {
		wp_nonce_field( 'cp_admin_nonce', 'cp_admin_wpnonce' );
	}

	/**
	 * Generates HTML content for payment methods
	 *
	 * @return string
	 */
	public function generate_private_key_html() {
		ob_start();
		?>

        <tr valign="top">
            <th scope="row" class="titledesc">
				<?php
				esc_html_e( 'Private Key Upload:', CpConstants::CP_TRANSLATIONS ); ?>
            </th>
            <td class="forminp" id="cp_private_key">
                <div>
                    <input id="cp_private_key_upload" name="cp_private_key_upload" type="file">
                </div>
            </td>
        </tr>
		<?php

		return ob_get_clean();
	}

	/**
	 * Generates HTML content for payment methods
	 *
	 * @return string
	 */
	public function generate_update_payment_providers_html() {
		ob_start();
		?>

        <style>
            .update_button_box {
                display: flex;
                flex-direction: row;
                align-items: center;
            }

            .update_button_box > .update_button_disclaimer, .update_button_api_warning {
                display: none;
                font-size: 0.75rem;
                margin-left: 5px;
                font-weight: bold;
                color: red;
            }
        </style>
        <tr valign="top">
            <th scope="row" class="titledesc">
				<?php
				esc_html_e( 'Update Payment Providers:', CpConstants::CP_TRANSLATIONS ); ?>
            </th>
            <td class="forminp" id="cp_update_payment_providers">
                <div class="update_button_box">
                    <button type="button" class="button-primary woocommerce-save-button"
                            id="cp_update_payment_providers_button">
                        Update
                    </button>
                    <div class="update_button_disclaimer">
						<?php
						echo esc_attr__( 'To update payment providers, make sure to first save the form after changing payment country',
							CpConstants::CP_TRANSLATIONS ); ?>
                    </div>
                    <div class="update_button_api_warning">
						<?php
						echo sprintf(esc_attr__( 'You should wait at least %s seconds to update payment providers',
							CpConstants::CP_TRANSLATIONS ), self::API_UPDATE_INTERVAL); ?>
                    </div>
                </div>
            </td>
        </tr>

        <script>
            jQuery(function ($) {
                const pathToScript = '/wp-content/plugins/connectpay-payment-gateway/includes/scripts/cp-redirect-before-endpoint.php?method=getProviders';

                $('#cp_update_payment_providers_button').on('click', function (e) {
                    const currentDate = new Date();
                    const currentDateInUTC = new Date(
                        currentDate.getUTCFullYear() + '-' + (currentDate.getUTCMonth() + 1) + '-' + currentDate.getUTCDate()
                        + ' ' + currentDate.getUTCHours() + ':' + currentDate.getUTCMinutes() + ':' + currentDate.getUTCSeconds()
                    );

                    const lastUpdateDate = new Date('<?php echo esc_attr( $this->last_update ); ?>');
                    const secondsElapsed = Math.floor((currentDateInUTC.getTime() - lastUpdateDate.getTime()) / 1000);

                    if (secondsElapsed < <?php echo esc_attr( self::API_UPDATE_INTERVAL ); ?>) {
                        e.preventDefault();
                        $('.update_button_api_warning').show();
                    } else {
                        window.location.href = window.location.protocol + '//' + window.location.host + pathToScript;
                    }
                });

                $('input[type="radio"][name="woocommerce_cp_payment_gateway_country"]').on('change', function () {
                    if ($(this).val() !== '<?php echo esc_attr( $this->payment_country ); ?>') {
                        $('#cp_update_payment_providers_button').prop('disabled', true);
                        $('.update_button_disclaimer').show();
                        $('.update_button_api_warning').hide();
                    } else {
                        $('#cp_update_payment_providers_button').prop('disabled', false);
                        $('.update_button_disclaimer').hide();
                    }
                });
            });
        </script>
		<?php

		return ob_get_clean();
	}

	/**
	 * Generates HTML content for payment methods
	 *
	 * @return string
	 */
	public function generate_last_update_html() {
		ob_start();

		// Also generate nonce hidden field
		$this->generate_cp_admin_nonce();

		$last_update = $this->last_update;
		?>
        <input type="hidden" id="cp_last_update" value="<?php
		echo esc_attr( $last_update ); ?>">
		<?php

		return ob_get_clean();
	}

	/**
	 * Generates HTML content for payment methods
	 *
	 * @return string
	 */
	public function generate_payment_methods_html() {
		ob_start();

		$methods             = $this->payment_methods;
		$no_methods_selected = true;
		foreach ( $methods as $method ) {
			if ( 'yes' === $method ) {
				$no_methods_selected = false;
			}
		}
		?>

        <style>
            .payment_methods_disclaimer {
                font-size: 0.8rem;
                color: red;
                font-weight: bold;
            }
        </style>
        <tr valign="top">
            <th scope="row" class="titledesc">
				<?php
				esc_html_e( 'Payment methods:', CpConstants::CP_TRANSLATIONS ); ?>
            </th>
            <td class="forminp" id="cp_payment_methods">
                <div class="checkbox-list">
                    <fieldset>
                        <legend class="screen-reader-text"><span></span></legend>
                        <label for="woocommerce_cp_payment_gateway_method_pis">
                            <input class="" type="checkbox"
                                   name="woocommerce_cp_payment_gateway_method_pis"
                                   id="woocommerce_cp_payment_gateway_method_pis"
                                   style="" value="1" data-section="cp_payment_method"
								<?php
								if ( 'yes' === $methods['woocommerce_cp_payment_gateway_method_pis'] ) : ?>
                                    checked="checked"
								<?php
								endif; ?>
                            >
							<?php
							echo esc_attr( self::PAYMENT_METHOD_PIS ); ?>
                        </label>
                        <br>
                    </fieldset>
                    <fieldset>
                        <legend class="screen-reader-text"><span></span></legend>
                        <label for="woocommerce_cp_payment_gateway_method_card">
                            <input class="" type="checkbox"
                                   name="woocommerce_cp_payment_gateway_method_card"
                                   id="woocommerce_cp_payment_gateway_method_card"
                                   style="" value="1" data-section="cp_payment_method"
								<?php
								if ( 'yes' === $methods['woocommerce_cp_payment_gateway_method_card'] ) : ?>
                                    checked="checked"
								<?php
								endif; ?>
                            >
							<?php
							echo esc_attr( self::PAYMENT_METHOD_CARD ); ?>
                        </label>
                        <br>
                    </fieldset>
                </div>
                <span class="payment_methods_disclaimer"
                      style="display:
				      <?php
				      if ( $no_methods_selected ) {
					      echo 'inline-block';
				      } else {
					      echo 'none';
				      }
				      ?>
                              "
                >
					<?php
					echo esc_attr__( 'At least one payment method should be selected', CpConstants::CP_TRANSLATIONS ); ?>
				</span>
            </td>
        </tr>

        <script>
			<?php if (! empty( $this->payment_providers_db )) : ?>
            jQuery(function ($) {
                if ($("input[data-section='cp_payment_method']:checked").length === 0) {
                    $(".woocommerce-save-button[value='Save changes']").prop('disabled', true);
                }

                $('input[data-section="cp_payment_method"]').on('change', function () {
                    var $isPisChecked = $('#woocommerce_cp_payment_gateway_method_pis').is(':checked');
                    var $isCardChecked = $('#woocommerce_cp_payment_gateway_method_card').is(':checked');

                    if ($("input[data-section='cp_payment_method']:checked").length === 0) {
                        $(".woocommerce-save-button[value='Save changes']").prop('disabled', true);
                        $('.payment_methods_disclaimer').show();
                    } else {
                        $('.payment_methods_disclaimer').hide();
                    }

                    if ($("input[data-section='cp_payment_provider']:checked").length === 0 && $isPisChecked) {
                        $(".woocommerce-save-button[value='Save changes']").prop('disabled', true);
                        $('.payment_providers_disclaimer').show();
                    }

                    if (($("input[data-section='cp_payment_provider']:checked").length === 0 && !$isPisChecked && $isCardChecked)
                        || ($("input[data-section='cp_payment_provider']:checked").length > 0
                            && $("input[data-section='cp_payment_method']:checked").length > 0)) {
                        $(".woocommerce-save-button[value='Save changes']").prop('disabled', false);
                        $('.payment_methods_disclaimer').hide();
                        $('.payment_providers_disclaimer').hide();
                    }
                });
            });
			<?php else : ?>
            jQuery(function ($) {
				<?php if ($no_methods_selected) : ?>
                $(".woocommerce-save-button[value='Save changes']").prop('disabled', true);
                $('.payment_methods_disclaimer').show();
				<?php endif; ?>

                $('input[data-section="cp_payment_method"]').on('change', function () {
                    if ($("input[data-section='cp_payment_method']:checked").length === 0) {
                        $(".woocommerce-save-button[value='Save changes']").prop('disabled', true);
                        $('.payment_methods_disclaimer').show();
                    } else {
                        $(".woocommerce-save-button[value='Save changes']").prop('disabled', false);
                        $('.payment_methods_disclaimer').hide();
                    }
                })
            });
			<?php endif; ?>
        </script>
		<?php

		return ob_get_clean();
	}

	/**
	 * Generates HTML content for payment providers
	 *
	 * @return string
	 */
	public function generate_payment_providers_html() {
		ob_start();

		$providers = $this->payment_providers_db;
		$country   = $this->payment_country;
		?>

        <style>
            .empty_payment_providers, .payment_providers_disclaimer {
                font-size: 0.85rem;
                color: red;
                font-weight: bold;
            }

            .provider-label {
                display: flex;
                flex-direction: column;
                text-align: center;
            }
        </style>
        <tr valign="top">
            <th scope="row" class="titledesc">
				<?php
				esc_html_e( 'Payment providers:', CpConstants::CP_TRANSLATIONS ); ?>
            </th>
            <td class="forminp" id="cp_payment_providers">
				<?php
				if ( empty( $providers ) ) : ?>
                    <div class="empty_payment_providers">Update payment providers</div>
				<?php
				else : ?>
                <span class="payment_providers_disclaimer" style="display: none">
						<?php
						echo esc_attr__( 'At least one payment provider should be selected', CpConstants::CP_TRANSLATIONS ); ?>
					</span>

                <div class="checkbox-list">
					<?php
					$maxcols = 5;
					$i       = 0;

					echo '<table>';
					echo '<tr>';
					foreach ( $providers

					as $provider ) :
					$provider_wc_id        = 'woocommerce_cp_payment_gateway_provider_' . $provider['id'];
					$provider_logo         = $provider['presentation']['logo'];
					$null_logo             = plugins_url() . '/connectpay-payment-gateway/assets/img/nulllogo.png';
					$logo                  = is_null( $provider_logo ) ? $null_logo : $provider_logo;
					$provider_country_code = $provider['countryCode'];
					$provider_name         = $provider['name'];

					if ( $i == $maxcols ) {
						$i = 0;
						echo '</tr><tr>';
					}

					if ( $provider_country_code === $country ) {
					?>
            <td>
                <fieldset>
                    <legend class="screen-reader-text"><span></span></legend>
                    <br>
                    <input class="" type="checkbox"
                           name="<?php
					       echo esc_attr( $provider_wc_id ); ?>" id="<?php
					echo esc_attr( $provider_wc_id ); ?>"
                           style="" value="1" data-section="cp_payment_provider"
						<?php
						if ( 'yes' === $providers[ $provider_wc_id ]['checked'] ) : ?>
                            checked="checked"
						<?php
						endif; ?>
                    >
                    <label for="<?php
					echo esc_attr( $provider_wc_id ); ?>" class="provider-label">
                        <img width="150px" height="80px" alt="Payment provider logo"
                             src="<?php
						     echo esc_attr( $logo ); ?>"/>
                        <div class="provider-name"><?php
							echo esc_attr( $provider_name ); ?></div>
                    </label>
                </fieldset>
            </td>
			<?php
			$i ++;
			}
			endforeach;

			while ( $i <= $maxcols ) {
				echo '<td>&nbsp;</td>';
				$i ++;
			}

			echo '</tr>';
			echo '</table>';
			?>
            </div>
			<?php
			endif; ?>
            </td>
        </tr>

        <script>
			<?php if (! empty( $providers )) : ?>
            jQuery(function ($) {
                if ($("input[data-section='cp_payment_provider']:checked").length === 0) {
                    $(".woocommerce-save-button[value='Save changes']").prop('disabled', true);
                }

                $('input[data-section="cp_payment_provider"]').on('change', function () {
                    var $isPisChecked = $('#woocommerce_cp_payment_gateway_method_pis').is(':checked');
                    var $isCardChecked = $('#woocommerce_cp_payment_gateway_method_card').is(':checked');

                    if ($("input[data-section='cp_payment_provider']:checked").length === 0 && $isPisChecked) {
                        $(".woocommerce-save-button[value='Save changes']").prop('disabled', true);
                        $('.payment_providers_disclaimer').show();
                    } else {
                        $('.payment_providers_disclaimer').hide();
                    }

                    if (($("input[data-section='cp_payment_provider']:checked").length === 0 && !$isPisChecked && $isCardChecked)
                        || ($("input[data-section='cp_payment_provider']:checked").length > 0
                            && $("input[data-section='cp_payment_method']:checked").length > 0)) {
                        $(".woocommerce-save-button[value='Save changes']").prop('disabled', false);
                        $('.payment_methods_disclaimer').hide();
                        $('.payment_providers_disclaimer').hide();
                    }
                });
            });
			<?php endif; ?>
        </script>
		<?php

		return ob_get_clean();
	}

	/**
	 * Generates HTML content for payment countries
	 *
	 * @return string
	 */
	public function generate_payment_country_html() {
		ob_start();

		$country          = $this->payment_country;
		$is_all_countries = $this->is_all_countries;
		?>

        <style>
            .germany_disclaimer {
                font-size: 0.75rem;
                color: red;
                margin-left: 6px;
            }
        </style>
        <tr valign="top">
            <th scope="row" class="titledesc">
				<?php
				esc_html_e( 'Payment countries:', CpConstants::CP_TRANSLATIONS ); ?>
            </th>
            <td class="forminp" id="cp_payment_country">
                <div class="checkbox-list">
                    <fieldset>
                        <legend class="screen-reader-text"><span></span></legend>
                        <label for="woocommerce_cp_payment_gateway_country_germany">
                            <input class="" type="radio"
                                   name="woocommerce_cp_payment_gateway_country"
                                   id="woocommerce_cp_payment_gateway_country_germany"
                                   style="" value="<?php
							echo esc_attr( self::PAYMENT_COUNTRY_CODE_GERMANY ); ?>"
								<?php
								if ( self::PAYMENT_COUNTRY_CODE_GERMANY === $country && false === $is_all_countries ) : ?>
                                    checked="checked"
								<?php
								endif; ?>
                            >
							<?php
							echo esc_attr( self::PAYMENT_COUNTRY_CODE_GERMANY ); ?>
                        </label>
                        <span class="germany_disclaimer"
                              style="display: <?php
						      echo ( self::PAYMENT_COUNTRY_CODE_GERMANY === $this->payment_country && ! $is_all_countries ) ? 'inline-block' : 'none'; ?>"
                        >
							<?php
							echo esc_attr__( '(Default ConnectPay integration, without payment provider selection in settings)',
								CpConstants::CP_TRANSLATIONS ); ?>
						</span>
                        <br>
                    </fieldset>
                    <fieldset>
                        <legend class="screen-reader-text"><span></span></legend>
                        <label for="woocommerce_cp_payment_gateway_country_finland">
                            <input class="" type="radio"
                                   name="woocommerce_cp_payment_gateway_country"
                                   id="woocommerce_cp_payment_gateway_country_finland"
                                   style="" value="<?php
							echo esc_attr( self::PAYMENT_COUNTRY_CODE_FINLAND ); ?>"
								<?php
								if ( self::PAYMENT_COUNTRY_CODE_FINLAND === $country && false === $is_all_countries ) : ?>
                                    checked="checked"
								<?php
								endif; ?>
                            >
							<?php
							echo esc_attr( self::PAYMENT_COUNTRY_CODE_FINLAND ); ?>
                        </label>
                        <br>
                    </fieldset>
                    <fieldset>
                        <legend class="screen-reader-text"><span></span></legend>
                        <label for="woocommerce_cp_payment_gateway_country_netherlands">
                            <input class="" type="radio"
                                   name="woocommerce_cp_payment_gateway_country"
                                   id="woocommerce_cp_payment_gateway_country_netherlands"
                                   style="" value="<?php
							echo esc_attr( self::PAYMENT_COUNTRY_CODE_NETHERLANDS ); ?>"
								<?php
								if ( self::PAYMENT_COUNTRY_CODE_NETHERLANDS === $country && false === $is_all_countries ) : ?>
                                    checked="checked"
								<?php
								endif; ?>
                            >
							<?php
							echo esc_attr( self::PAYMENT_COUNTRY_CODE_NETHERLANDS ); ?>
                        </label>
                        <br>
                    </fieldset>
                    <fieldset>
                        <legend class="screen-reader-text"><span></span></legend>
                        <label for="woocommerce_cp_payment_gateway_country_lithuania">
                            <input class="" type="radio"
                                   name="woocommerce_cp_payment_gateway_country"
                                   id="woocommerce_cp_payment_gateway_country_lithuania"
                                   style="" value="<?php
							echo esc_attr( self::PAYMENT_COUNTRY_CODE_LITHUANIA ); ?>"
								<?php
								if ( self::PAYMENT_COUNTRY_CODE_LITHUANIA === $country && false === $is_all_countries ) : ?>
                                    checked="checked"
								<?php
								endif; ?>
                            >
							<?php
							echo esc_attr( self::PAYMENT_COUNTRY_CODE_LITHUANIA ); ?>
                        </label>
                        <br>
                    </fieldset>
                    <fieldset>
                        <legend class="screen-reader-text"><span></span></legend>
                        <label for="woocommerce_cp_payment_gateway_country_all">
                            <input class="" type="radio"
                                   name="woocommerce_cp_payment_gateway_country"
                                   id="woocommerce_cp_payment_gateway_country_all"
                                   style="" value="<?php
							echo esc_attr( self::PAYMENT_COUNTRY_CODE_ALL ); ?>"
								<?php
								if ( true === $is_all_countries ) : ?>
                                    checked="checked"
								<?php
								endif; ?>
                            >
							<?php
							echo esc_attr__( self::PAYMENT_COUNTRY_TITLE_FOR_ALL, CpConstants::CP_TRANSLATIONS ); ?>
                        </label>
                        <br><br>
                        <div class="all_countries_sub_select"
                             style="
						     <?php
						     if ( ! $this->is_all_countries ) {
							     echo 'display: none';
						     }
						     ?>
                                     "
                        >
                            <span><strong><?php
		                            echo esc_attr__( 'Default payment country in widget:', CpConstants::CP_TRANSLATIONS ); ?></strong></span>
                            <br><br>
                            <select name="woocommerce_cp_payment_gateway_sub_country">
                                <option value="<?php
								echo esc_attr( self::PAYMENT_COUNTRY_CODE_GERMANY ); ?>"
									<?php
									if ( self::PAYMENT_COUNTRY_CODE_GERMANY === $country ) : ?>
                                        selected="true"
									<?php
									endif; ?>
                                >
									<?php
									echo esc_attr( self::PAYMENT_COUNTRY_CODE_GERMANY ); ?>
                                </option>
                                <option value="<?php
								echo esc_attr( self::PAYMENT_COUNTRY_CODE_FINLAND ); ?>"
									<?php
									if ( self::PAYMENT_COUNTRY_CODE_FINLAND === $country ) : ?>
                                        selected="true"
									<?php
									endif; ?>
                                >
									<?php
									echo esc_attr( self::PAYMENT_COUNTRY_CODE_FINLAND ); ?>
                                </option>
                                <option value="<?php
								echo esc_attr( self::PAYMENT_COUNTRY_CODE_NETHERLANDS ); ?>"
									<?php
									if ( self::PAYMENT_COUNTRY_CODE_NETHERLANDS === $country ) : ?>
                                        selected="true"
									<?php
									endif; ?>
                                >
									<?php
									echo esc_attr( self::PAYMENT_COUNTRY_CODE_NETHERLANDS ); ?>
                                </option>
                                <option value="<?php
								echo esc_attr( self::PAYMENT_COUNTRY_CODE_LITHUANIA ); ?>"
									<?php
									if ( self::PAYMENT_COUNTRY_CODE_LITHUANIA === $country ) : ?>
                                        selected="true"
									<?php
									endif; ?>
                                >
									<?php
									echo esc_attr( self::PAYMENT_COUNTRY_CODE_LITHUANIA ); ?>
                                </option>
                            </select>
                        </div>
                        <br>
                    </fieldset>
                </div>
            </td>
        </tr>

        <script>
            jQuery(function ($) {
                $('input[type="radio"][name="woocommerce_cp_payment_gateway_country"]').on('change', function () {
                    if ($(this).val() === '<?php echo esc_attr( self::PAYMENT_COUNTRY_CODE_GERMANY ); ?>') {
                        $('.germany_disclaimer').show();
                    } else {
                        $('.germany_disclaimer').hide();
                    }

                    if ($(this).val() === '<?php echo esc_attr( self::PAYMENT_COUNTRY_CODE_ALL ); ?>') {
                        $('.all_countries_sub_select').show();
                    } else {
                        $('.all_countries_sub_select').hide();
                    }
                });
            });
        </script>
		<?php

		return ob_get_clean();
	}

	/**
	 * Save payment methods checkboxes
	 */
	public function save_payment_methods() {
		// Verify nonce in one place
		if ( isset( $_POST['cp_admin_wpnonce'] ) && wp_verify_nonce( sanitize_key( $_POST['cp_admin_wpnonce'] ), 'cp_admin_nonce' ) ) {
			$methods = [];

			$pis  = isset( $_POST['woocommerce_cp_payment_gateway_method_pis'] ) && ! empty( sanitize_text_field( $_POST['woocommerce_cp_payment_gateway_method_pis'] ) )
				? sanitize_text_field( $_POST['woocommerce_cp_payment_gateway_method_pis'] ) : null;
			$card = isset( $_POST['woocommerce_cp_payment_gateway_method_card'] ) && ! empty( sanitize_text_field( $_POST['woocommerce_cp_payment_gateway_method_card'] ) )
				? sanitize_text_field( $_POST['woocommerce_cp_payment_gateway_method_card'] ) : null;

			$pis  = $this->get_new_checkbox_value( $pis );
			$card = $this->get_new_checkbox_value( $card );

			$methods['woocommerce_cp_payment_gateway_method_pis']  = $pis;
			$methods['woocommerce_cp_payment_gateway_method_card'] = $card;

			update_option( 'cp_payment_methods', $methods );
		}
	}

	/**
	 * Save payment providers checkboxes
	 */
	public function save_payment_providers() {
		if ( isset( $_POST['cp_admin_wpnonce'] ) && wp_verify_nonce( sanitize_key( $_POST['cp_admin_wpnonce'] ), 'cp_admin_nonce' ) ) {
			$providers = $this->payment_providers_db;
			foreach ( $providers as $provider_id => $provider ) {
				$provider_checkbox_value = isset( $_POST[ $provider_id ] ) &&
				                           ! is_null( sanitize_text_field( $_POST[ $provider_id ] ) ) ?
					sanitize_text_field( $_POST[ $provider_id ] ) : null;

				$provider_checkbox_value              = $this->get_new_checkbox_value( $provider_checkbox_value );
				$providers[ $provider_id ]['checked'] = $provider_checkbox_value;
			}

			update_option( 'cp_payment_providers', $providers );
		}
	}

	/**
	 * Save payment countries checkboxes
	 */
	public function save_payment_country() {
		if ( isset( $_POST['cp_admin_wpnonce'] ) && wp_verify_nonce( sanitize_key( $_POST['cp_admin_wpnonce'] ), 'cp_admin_nonce' ) ) {
			if ( isset( $_POST['woocommerce_cp_payment_gateway_country'] ) && ! empty( $_POST['woocommerce_cp_payment_gateway_country'] ) ) {
				$selected_country = sanitize_text_field( $_POST['woocommerce_cp_payment_gateway_country'] );
			}

			$is_all_countries = self::PAYMENT_COUNTRY_CODE_ALL === $selected_country;
			update_option( 'cp_is_all_countries', $is_all_countries );

			if ( $is_all_countries ) {
				if ( isset( $_POST['woocommerce_cp_payment_gateway_sub_country'] ) && ! empty( $_POST['woocommerce_cp_payment_gateway_sub_country'] ) ) {
					$selected_country = sanitize_text_field( $_POST['woocommerce_cp_payment_gateway_sub_country'] );
				}
				update_option( 'cp_payment_country', $selected_country );
				update_option( 'cp_payment_providers', [] );
			} else {
				if ( $this->payment_country !== $selected_country ) {
					update_option( 'cp_payment_providers', [] );
				}

				update_option( 'cp_payment_country', $selected_country );
			}
		}
	}

	public function handle_file_upload() {
		$certificate_formats = [
			'p8',
			'key',
			'p10',
			'csr',
			'crl',
			'pem',
			'crt',
			'p7b',
			'spc',
			'p7r',
			'p7s',
			'der',
			'cer',
			'pfx',
			'p12',
		];

		if ( ! empty( $_FILES['cp_private_key_upload']['tmp_name'] ) ) {
			if ( ! function_exists( 'wp_handle_upload' ) ) {
				require_once( ABSPATH . 'wp-admin/includes/file.php' );
			}
			$urls            = wp_handle_upload( $_FILES['cp_private_key_upload'], [ 'test_form' => false ] );
			$file_path       = $urls['file'];
			$file_path_array = explode( '/', $file_path );
			$file_extension  = explode( '.', $file_path_array[ count( $file_path_array ) - 1 ] )[1];

			// Check if file extension is valid for certificate
			if ( ! in_array( $file_extension, $certificate_formats ) ) {
				add_action( 'admin_notices', 'wrong_extension' );
			} else {
				// Encode private key with Base64 encoding
				$encoded_private_key = base64_encode( file_get_contents( $file_path ) );
				update_option( 'cp_private_key', $encoded_private_key );
			}

			wp_delete_file( $file_path );
		}

		function wrong_extension() {
			?>
            <div class="error notice">
                <p><strong><?php
						esc_html_e( 'Wrong private key extension.', CpConstants::CP_TRANSLATIONS ); ?></strong></p>
            </div>
			<?php
		}
	}

	/**
	 * Get "yes" or "no" value, based on checkbox state
	 *
	 * @param string|null $value
	 *
	 * @return string
	 */
	private function get_new_checkbox_value( $value ) {
		return is_null( $value ) ? self::CHECKBOX_FALSE : self::CHECKBOX_TRUE;
	}

	private function get_option_bool_value( $value ) {
		return $value ? '1' : '0';
	}

}
